*! version 1.01 94/9/26; version2.0 2020/06/30; 2020/07/02
program define l2cvwarpy
  version 16.0
*First written 94/9/26; Revisions: 95/03/07; 2020/06/30; 2020/07/02
*Authors: Salgado-Ugarte, I.H., M. Shimizu, T. Taniuchi and
*V.M. Saito-Quezada.
*Colaborators (Python): Noel Plascencia Díaz, M.M. Salgado-Saito
*This program calculates least squares crossvalidation for warping
*density estimation using a Phyton script program which utilizes
*modified algorithms from: 
*Haerdle W. (1991) "Smoothing Techniques with Implementations in S", 
*Springer-Verlag Series in Statistics, New York
*Salgado-Ugarte, I.H. (2002) Suavización no paramética para análisis
*de datos. FES-Zaragoza y DGAPA, UNAM, México 
*Kernel codes:
*1 Uniform
*2 Triangle
*3 Epanechnikov
*4 Quartic (Biweight)
*5 Triweight
*6 Gaussian
local varlist "req ex min(1) max(1)"
local if "opt"
local in "opt"
#delimit ;

local options "Delta(real 0) Kercode(integer 0) MSTart(integer 1)
     MENd(integer 0) Gen(string) noGraph T1title(string) MSymbol(string)
     Connect(string) *";

#delimit cr
parse "`*'"
parse "`varlist'", parse(" ")
quietly {
preserve
  if "`gen'"~="" {
     tempfile _data
     save `_data'
     }
  local deltav=`delta'
  local kco=`kercode'
  summ `1'
  sort `1' 
  if "`mstart'"~="" {
    local mst=`mstart'
  }
  if "`mend'"~="" {
    local men=`mend'
  }

tempvar xvar
gen `xvar'=`1' `if' `in'
drop if `xvar'==.
if `xvar'[1]==. {
   di in red "no observations"
   exit
   }
outfile `xvar' using _data2, replace
tempvar dval kc msv mev
gen `dval'=`deltav'
if `dval'==0 {
   di in red "you must provide the delta value"
   exit
   }
gen `kc'=`kco'
if `kc'==0 {
     di in red "you must provide the kernel code"
     exit
	 }

if `kc'<1 | `kc'>6 {
   di in red "invalid kernel code"
   exit
   }
gen `msv'=`mst'
if `msv'<1 {
   di in red "you must use a Mstart > 0"
   exit
   }
gen `mev'=`men'
replace `mev'=int(0.33*((_result(6)-_result(5))/`delta')) if `men'==0
keep `dval' `kc' `msv' `mev'
drop if _n>1
set obs 1
outfile using _inpval, replace
drop _all
*!l2cvwarpy
python script l2cvwarpy.py
!del _data2.raw
!del _inpval.raw 
tempvar num mval cv hval
import delimited `num' `cv' `mval' `hval' using resfile.csv
label variable `mval' "M-value"
label variable `cv' "CV-value"
label variable `hval' "Bandwidth"
!del resfile.csv
if "`graph'" ~= "nograph"  {
   if "`t1title'" ==""{
         local t1title "L2 Cross Validation (WARP)"
         local t1title "`t1title', delta = `deltav', Kernel = `kco'"
   }
   if "`msymbol'"=="" { 
   	local msymbol "i" 
   }
   if "`connect'"=="" { 
   	local connect "l" 
   }

   scatter `cv' `mval', `options' t1("`t1title'") /*
                          */ ms(`msymbol') c(`connect')
*   graph `cv' `hval', `options' t1("`t1title'") /*
*                          */ s(`symbol') c(`connect')
}

sort `cv'
local i = 1
noi di as text _newline _dup(79) "-"

local title "Least Squares Cross-validation for WARPing density estimation"
if `kco'==1 {
   local kerlab "Uniform kernel"
   }
else if `kco'==2 {
   local kerlab "Triangle kernel"
   }
else if `kco'==3 {
   local kerlab "Epanech. kernel"
   }
else if `kco'==4 {
   local kerlab "Quartic kernel"
   }
else if `kco'==5 {
   local kerlab "Triweight kernel"
   }
else {
   local kerlab "Gaussian kernel"
   }
*local title "`title', `kerlab'"
noi di as text "`title', " as res "`kerlab'"

noi di as text _dup(79) "-"
while `i'<6 {
noi di as text "CV-value = " as res %10.8f `cv'[`i'] /*
     */as text _column(30) "M-value = " as res %5.0g `mval'[`i'] /*
	 */as text _column(50) "Bandwidth = " as res %8.4f `hval'[`i']
      local i = `i'+1
   }
if "`gen'"~="" {
   restore, not
   merge using `_data'
   drop _merge
   parse "`gen'", parse(" ")  
   gen `1'=`cv'
   gen `2'=`mval'
   gen `3'= `hval'
   }
}
end
