{smcl}
{* *! version 1.0.0 24/jul2020}{...}
{hline}
{cmd:help bcvwarpy}                                [STB-27: snp6.2 (original version)]
{hline}

{title:Title}

{p2colset 5 18 20 2}{...}
{p2col :{hi:bcvwarpy} {hline 2}}Biased cross-validation for kernel bandwidth selection
 (Python script version){p_end}
{p2colreset}{...}

{title:Syntax}
{p 8 17 2}
{cmd:bcvwarpy} {varname} {ifin} {cmd:,} {cmdab:d:elta(}{it:#}{cmd:)}
{cmdab:k:ercode(}{it:#}{cmd:)}
[{cmdab:mst:art(}{it:#}{cmd:)}
{cmdab:men:d(}{it:#}{cmd:)}
{cmdab:g:en(}{it:cvval mval hval}{cmd:)}
{cmdab:nog:raph}
{cmd:graph_options}]

{title:Description}

{pstd}{cmd:bcvwarpy} calculates biased cross-validation by means of the ASH-WARPing
procedure (Scott and Terrell, 1987; Haerdle, 1991) draws a graph as an aid
to locate visually the minimum, and produces a table with the five lowest
biased-cross-validation scores with the corresponding M and h (bandwidth)
values.

{title:Options}


{phang}{opt delta}({it:#}) specifies the small bandwidth resulting from the shifting of
   histograms to average. This value is interpreted as the measurement accuracy
   of the observations which are rounded to the level it indicates.

{phang}{opt kercode}({it:#}) permits to choose the weight function according to the following
   codes:

   1 = Quartic (Biweight)
   2 = Triweight


{phang}{opt mstart}({it:#}) and {opt mend}({it:#}) allow to determine the range of M considered for
   the minimum search. The default mstart is 1; if mend is not specified then
   it is set to about a third of the range of the observations which in general
   produces an adequate initial estimation.

{phang}{opt gen}({it:bcvval mval hval}) permits to create three variables containing
   respectively the estimated biased cross-validation, M, and h(bandwidth)
   values.

{phang}{opt nograph} suppresses the graph drawing.

{phang}{opt graph_options} are any of the options allowed with {hi:graph, twoway}.


{title:Remarks}

{hi:d}elta, and {hi:k}ercode, are not optional. If the user does not provide them,
the program halts and displays an error message.

This program uses a Python script for kernel density estimation and cross-validation
calculations based on the algorithms and programs provided by Haerdle (1991) and 
Salgado-Ugarte (1995; 2002).

It is adviced to use a semilogarithmic scale by using the {hi:xscale(log)} graphic option,
and to include the oversmoothed bandwidth (Scott, 1992) calculated with the
{cmd bandw} command (rescaled if necessary) by means of the {hi:xline()} option.

With this command, it is possible to estimate the "optimal" smoothing parameter
for a wide set of kernel density estimators. Specifying the triangular kernel
it is possible to obtain the optimal binwidth for the ASH, and (increasing M in
the WARPing density estimation) the corresponding triangle kernel estimator.
The other weight functions apply accordingly to their respective density
estimators.

{title:Examples}

{phang}{stata "use bufsnow" :. use bufsnow}{p_end}

{phang}{stata "bcvwarpy snow, delta(1) kercode(2) mstart(15) me(25) xscale(log) xline(35.29)" :. bcvwarpy snow, delta(1) kercode(2) mstart(15) me(25) xscale(log) xline(35.29)}{p_end}

{phang}{stata "bcvwarpy snow, d(1) k(1) ms(5) me(25) gen(cvval mval bwidth) nograph" :. bcvwarpy snow, d(1) k(1) ms(5) me(25) gen(cvval mval bwidth) nograph}{p_end}


{title:References}

   Haerdle, W. (1991) Smoothing Techniques with Implementation in S.
        Springer-Verlag Chapter 2: 43-84; Chapter 4: 90-119.
   Salgado-Ugarte, I.H. (1995) Nonparametric methods for fisheries data analysis
        and their application in conjunction with other statistical techniques
        to study biological data of the Japanese sea bass {it:Lateolabrax japonicus}
        in Tokyo Bay. PhD thesis, University of Tokyo, Japan: 389 p.
   Salgado-Ugarte, I.H. (2002) Suavización no paramétrica para análisis de datos.
        DGAPA y FES Zaragoza, UNAM, México: 139 p.   
   Salgado-Ugarte, I.H., M. Shimizu, and T. Taniuchi (1993) snp6: Exploring
        the shape of univariate data using kernel density estimators. Stata 
        Technical Bulletin 16: 8-19.
   Salgado-Ugarte, I.H., M. Shimizu, and T. Taniuchi (1995) snp6.1: ASH,
        WARPing, and kernel density estimation for univariate data. Stata
        Technical Bulletin XX: xx-xx.
   Salgado-Ugarte, I.H., M. Shimizu, and T. Taniuchi (199x) snp6.2: Practical
        Rules for bandwidth selection in univariate density estimation. Stata
       Technical Bulletin XX: xx-xx.
   Scott, D.W. (1992) Multivariate Density Estimation: Theory, Practice,
        and Visualization. John Wiley Chapters 3-6: 47-193.
   Scott, D.W. and G.R. Terrell, 1987. Biased and unbiased cross-validation in
        density estimation. Journal of the American Statistical Association, 
        82(400):1131-1146.
   Silverman, B.W. (1986) Density Estimation for Statistics and Data 
        Analysis. Chapman and Hall.


{title:Authors}

Original version:
Isaias H. Salgado-Ugarte, Makoto Shimizu and Toru Taniuchi
University of Tokyo, Faculty of Agriculture,
Department of Fisheries, Yayoi 1-1-1, Bunkyo-ku
Tokyo 113, Japan.(Fax 81-3-3812-0529)
Updated version:
Isaías H. Salgado-Ugarte & V. Mitsui Saito-Quezada
Colaborators (Python): N.I. Plascencia-Díaz & M.M. Salgado-Saito
Laboratorio de Biometría y Biología Pesquera
Facultad de Estudios Superiores Zaragoza
Universidad Nacional Autónoma de México
isalgado@unam.mx

{title:Also see}

{psee}

    STB: snp6 (STB-16); snp6.1 (STB-26); snp6.2 (STB-27), snp6.4 (STB-38)
{phang}On-line: {hi:help} for {help bandw1}, {help kerneld}, {help warpdenpy}, {help l2cvwarpy}, {help kernreg}
{p_end}
