*! version 1.00 94/11/14; version 2.00 2020/10/07
program define gwarprpy
  version 16.0
*First written 94/11/14; Revisions: 95/03/04; 2020/10/07; 30/03/2021
*Authors: Isaias H. Salgado-Ugarte, Makoto Shimizu,
*Toru Taniuchi and V. Mitsui Saito-Quezada.
*Colaborators (Python): Noel Plascencia Díaz, M.M. Salgado-Saito
*This program recall the Adjusted Prediction Error for the
*Haerdle WARPing approximation of the NADARAYA-WATSON regression
*estimate using the results of the Python executable program 
*gwarpreg.exe which utilizes modified algorithms and C programs from:
*Haerdle W (1991) *"Smoothing Techniques with Implementations in S", 
*Springer-Verlag Series in Statistics, New York.
*Salgado-Ugarte, I.H. (2002) Suavización no paramética para análisis
*de datos. FES-Zaragoza y DGAPA, UNAM, México 
local varlist "req ex min(2) max(2)"
local if "opt"
local in "opt"
#delimit ;
local options "Delta(real 0) Selec(integer 0) Kercod(integer 0)
       MSTart(integer 5) MENd(integer 0) BOUnd(real 0.1) Gen(string)
	   noGraph T1title(string) MSymbol(string) Connect(string) 
	   GRSel(integer 0) *";
#delimit cr
parse "`*'"
parse "`varlist'", parse(" ")
quietly {
preserve
if "`gen'"~="" {
   tempfile _data
   save `_data'
   }
  local delta=`delta'
  if "`mstart'"~="" {
    local ms=`mstart'
  }
  if "`mend'"~="" {
    local me=`mend'
  }
  if "`bound'"~="" {
    local bou=`bound'
  }

  local kco=`kercod'
  local selc=`selec'
  
tempvar yvar xvar
gen `yvar'=`1' `if' `in'
gen `xvar'=`2' `if' `in'
replace `xvar'=. if `yvar'==.
if `xvar'[1]==. {
   di in red "no observations"
   exit
   }
summ `xvar'
sort `xvar'
outfile `xvar' `yvar' using _data2 if `xvar' !=., replace
tempvar dval sval kc msv mev bouv
gen `dval'=`delta'
gen `sval'=`selec'

if `dval'==0 {
     di in red "you must provide delta (small bin width)"
     exit
	 }
if `sval'==0 {
     di in red "you must provide the selector function code"
     exit
	 }

if `sval'<1 | `sval'>5 {
   di in red "invalid selector code"
   exit
   }
gen `kc'=`kercod'

if `kc'==0 {
     di in red "you must provide the kernel code"
     exit
	 }

if `kc'<1 | `kc'>6 {
   di in red "invalid kernel code"
   exit
   }
gen `msv'=`ms'
if `msv'<5 {
   di in red "you must use a Mstart > 5"
   exit
   }
gen `mev'=`me'
replace `mev'=int(0.33*((_result(6)-_result(5))/`delta')) if `me'==0
gen `bouv'=`bou'
keep `dval' `sval' `kc' `msv' `mev' `bouv'
drop if _n>1
set obs 1
outfile using _inpval, replace
drop _all
python script gwarpreg.py
!del _data2.raw
!del _inpval.raw 
tempvar num score1 mval hval
import delimited `num' `score1' `mval' `hval' using resfile.csv, clear
label variable `mval' "M-value"
label variable `score1' "Score value"
label variable `hval' "Bandwidth"
*noi list `num' `score1' `mval' `hval'
!del resfile.csv

if `selc'==1 {
   local selab "Shibata"
   }
else if `selc'==2 {
   local selab "GCV"
   }
else if `selc'==3 {
   local selab "Akaike"
   }
else if `selc'==4 {
   local selab "FPE"
   }
else if `selc'==5 {
   local selab "Rice's T"
   }

if `kco'==1 {
   local kerlab "Uniform kernel"
   }
else if `kco'==2 {
   local kerlab "Triangle kernel"
   }
else if `kco'==3 {
   local kerlab "Epanech. kernel"
   }
else if `kco'==4 {
   local kerlab "Quartic kernel"
   }
else if `kco'==5 {
   local kerlab "Triweight kernel"
   }
else {
   local kerlab "Gaussian kernel"
   }   
   
if "`graph'" ~= "nograph"  {
   if "`t1title'" ==""{
         local t1title "G(M) CV (WARP)"
         local t1title "`t1title', Penalizing function = `selab', Kernel = `kco'"
   }
   if "`msymbol'"=="" { 
   	local msymbol "i" 
   }
   if "`connect'"=="" { 
   	local connect "l" 
   }

      if `grsel'==0 {
   scatter `score1' `mval', `options' t1("`t1title'") /*
                          */ ms(`msymbol') c(`connect') 
   }					  
   else if `grsel'==1 {
   scatter `score1' `hval', `options' t1("`t1title'") /*
          */ ms(`msymbol') c(`connect') 		  	  
   } 
   else if `grsel'>1 {
   di in red "invalid graph selector"
   exit	
   }
*   scatter `score1' `mval', `options' t1("`t1title'") /*
*                          */ ms(`msymbol') c(`connect') || /*
*	*/scatter `score1' `hval', `options' t1("`t1title'") /*
*          */ ms(`msymbol') c(`connect') xaxis(2) legend(off) 
}

*scatter `score1' `hval', c(l) ms(i)

sort `score1'
local i = 1
noi di as text _newline _dup(70) "-"
noi di as text "G(M) CV (WARP) N-W regression, PF = " /*
     */ as res "`selab'" as text ", Kernel = "as res "`kco'" /*
	 */ as text ", Delta = " %6.3f as res "`delta'"
noi di as text _dup(70) "-"
while `i'<6 {
noi di as text "M-value = " as res `mval'[`i'] _column(20) as text "Score value = " as res `score1'[`i'] _column(50) as text "Bandwidth = " %8.4f as res `hval'[`i']
      local i = `i'+1
   }
if "`gen'"~="" {
   restore, not
   merge using `_data'
   drop _merge
   parse "`gen'", parse(" ")  
   gen `1'=`score1'
   gen `2'=`mval'
   gen `3'= `hval'
   }
}
end
