{smcl}
{* *! version 1.0.0 25/jul2020}{...}
{hline}
{cmd:help gwarprpy}                                [STB-30: snp10 (original version)]
{hline}

{title:Title}

{p2colset 5 18 20 2}{...}
{p2col :{hi:gwarprpy} {hline 2}}Adjusted prediction error for WARPing regression (Nadaraya-Watson estimator)
 (Python script version){p_end}
{p2colreset}{...}

{title:Syntax}
{p 8 17 2}
{cmd:gwarprpy} {xvar yvar} {ifin} {cmd:,} {cmdab:d:elta(}{it:#}{cmd:)}
{cmdab:s:elec(}{it:#}{cmd:)}
{cmdab:k:ercode(}{it:#}{cmd:)}
[{cmdab:mst:art(}{it:#}{cmd:)}
{cmdab:men:d(}{it:#}{cmd:)}
{cmdab:bou:nd(}{it:#}{cmd:)}
{cmdab:g:en(}{it:cvval mval hval}{cmd:)}
{cmdab:grs:el(}{it:#}{cmd:)}
{cmdab:nog:raph}
{cmd:graph_options}]

{title:Description}

{pstd}{cmd:gwarprpy} calculates the adjusted prediction error of the WARPing
approximation of the Nadaraya-Watson with several penalizing functions in order
to find an optimal bandwidth for kernel regression. As a default this command
draws a graph of score vs. mval but the user may shift to the score vs. bandwidth
graph with the option {opt grsel}, and displays a table with the five minimal values
for the score and corresponding mval and bandwidths.

{title:Options}

{phang}{opt delta}({it:#}) specifies the desired accuracy for estimation (equivalent to the
    small bin width in WARPing density estimation).

{phang}{opt select{({it:#}) refers to the penalizing function according to the following
    numeric selector codes:

	1 = Shibata's model selector
	2 = Generalized cross-validation
	3 = Akaike's information criterion
	4 = Finite prediction error
	5 = Rice's T

{phang}{opt kercode}({it:#}) specifies the weight function (kernel) to calculate the
    required univariate densities according to the following numerical codes:

	1 = Uniform
 	2 = Triangle 
 	3 = Epanechnikov
 	4 = Quartic (Biweight)
 	5 = Triweight
 	6 = Gaussian

{phang}{opt mstart}({it:#}) permits to specify the initial value for the range of mval
    (equivalent to the bandwidth) used for the score minimum search. As a
    default {hi:ms}tart = 5.

{phang}{opt mend}({it:#}) permits to specify the final value for the range of mval used for
    the score minimum search.
    As a default {hi:me}nd = int(0.3*(max(x)-min(x)/delta).

{phang}{opt bound}({it:#}) specifies the fraction of the data to be excluded for the
    estimation. By default it is set to 0.1.

{phang}{opt gen}({it:score mvalue bandwidth}) permits to store the adjusted prediction
    error (score), and corresponding mval and bandwidth values.

{phang}{opt grsel}({it:#}) permits to chose between score vs. mval ({it:#} = 0, default) or 
     score vs hval graph ({it:#} = 1).

{phang}{opt nograph} suppresses the graph drawing.

{phang}{opt graph_options} are any of the options allowed with {hi:graph, twoway}.

{title:Remarks}

{hi:d}elta, {hi:s}elec, and {hi:k}ercode, are not optional. If the user does not provide
them, the program halts and displays an error message on screen.

This program uses ado and a Python script files for kernel density estimation and cross-validation 
calculations based on the programs presented in Salgado-Ugarte, et al. (1996) and it 
is based on the algorithms and programs provided by Haerdle (1991), Scott (1992; 2015) 
and Salgado-Ugarte (1995; 2002).

The resulting graphs and table are used to find and select the intervals
containing minimal score values by a trial and error procedure varying the
values for {hi:d}elta, {hi:mst}art (restricted to those >5) and {hi:men}d.


{title:Examples}

{phang}{stata "use geyser" :. use geyser}{p_end}

{phang}{stata "gwarprpy wait dura, delta(0.05) selec(2) kercode(4) mend(40)" :. gwarprpy wait dura, delta(0.05) selec(2) kercode(4) mend(40)}{p_end}

Will estimate the adjusted prediction error with an accuracy of 0.05, the
generalized cross-validation penalizing function, the Quartic kernel, and
an M range from 5 to 40.

{phang}{stata "use motcyc" :. use motcyc}{p_end}

{phang}{stata "gwarprpy accel time, d(0.2) s(1) k(4) me(20) gen(score mvalue bandw) grs(1)" :. gwarprpy accel time, d(0.2) s(1) k(4) me(20) gen(score mvalue bandw)}{p_end}

Besides the graphs and table, the program will generate three variables (score,
mvalue and bandw) with the score, mval and bandwidth values respectively.


{title:References}

   Haerdle, W. (1991) Smoothing Techniques with Implementation in S.
        Springer-Verlag Chapter 2: 43-84; Chapter 5: 121-149.
   Salgado-Ugarte, I.H. (1995) Nonparametric methods for fisheries data analysis
        and their application in conjunction with other statistical techniques
        to study biological data of the Japanese sea bass {it:Lateolabrax japonicus}
        in Tokyo Bay. PhD thesis, University of Tokyo, Japan: 389 p.
   Salgado-Ugarte, I.H. (2002) Suavización no paramétrica para análisis de datos.
        DGAPA y FES Zaragoza, UNAM, México: 139 p.   
   Salgado-Ugarte, I.H., M. Shimizu, and T. Taniuchi (1993) snp6: Exploring
        the shape of univariate data using kernel density estimators. Stata 
        Technical Bulletin 16: 8-19.
   Salgado-Ugarte, I.H., M. Shimizu, and T. Taniuchi (1995a) snp6.1: ASH,
        WARPing, and kernel density estimation for univariate data. Stata
        Technical Bulletin 26: 23-31.
   Salgado-Ugarte, I.H., M. Shimizu, and T. Taniuchi (1995b) snp6.2: Practical
        Rules for bandwidth selection in univariate density estimation. Stata
        Technical Bulletin 27: 5-19.
   Salgado-Ugarte, I.H., M. Shimizu, and T. Taniuchi (1996) snp10: Nonparametric 
        regression: Kernel, WARP and k-NN estimators. Stata Technical Bulletin 30: 15-30.
   Scott, D.W. (1992) Multivariate Density Estimation: Theory, Practice,
        and Visualization. John Wiley Chapter 6: 125-143; Chapter 8: 219-245.
   Scott, D.W. (2015) Multivariate Density Estimation: Theory, Practice,
        and Visualization. 2nd. ed. John Wiley Chapter 6: 137-216; Chapter 8: 241-266.   
   Silverman, B.W. (1986) Density Estimation for Statistics and Data 
        Analysis. Chapman and Hall.


{title:Authors}

Original version:
Isaias H. Salgado-Ugarte, Makoto Shimizu and Toru Taniuchi
University of Tokyo, Faculty of Agriculture,
Department of Fisheries, Yayoi 1-1-1, Bunkyo-ku
Tokyo 113, Japan.(Fax 81-3-3812-0529)
Updated version:
Isaías H. Salgado-Ugarte & V. Mitsui Saito-Quezada
Colaborators (Python): N.I. Plascencia-Díaz & M.M. Salgado-Saito
Laboratorio de Biometría y Biología Pesquera
Facultad de Estudios Superiores Zaragoza
Universidad Nacional Autónoma de México
isalgado@unam.mx

{title:Also see}

{psee}

    STB: snp6 (STB-16); snp6.1 (STB-26); snp6.2 (STB-27); snp6.4 (STB-38); snp10 (STB-30)
{phang}On-line: {hi:help} for {help kernreg}, {help warpregpy}, {help warpdenpy}
{p_end}
