#!/usr/bin/env python
# coding: utf-8

# In[1]:


import numpy as np
import pandas as pd


# # Import Data

# In[2]:


data = np.loadtxt('_data2.raw')


# In[3]:


n = len(data)


# # Model Parameters

# In[4]:

params = np.loadtxt('_inpval.raw')

h = float(params[0])
M = int(params[1])
kernel_type = int(params[2])


# # Initial Calc

# In[5]:


maxval = data[-1];
minval = data[0];

if kernel_type == 6:
    h = 4*h

delta = h/M
numbin = int((maxval-minval)/delta)+2*(M+1+round((M/10)+0.5))
start = (minval-h)-delta*0.1

if start<0:
    origin = (round((start/delta)-0.5)-0.5)*delta

else:
    origin = (int(start/delta)-0.5)*delta

if n+1<numbin:
    maxnoemp = n+1

else:
    maxnoemp = numbin

binumber = 0
index = 0

binmesh = np.zeros(numbin+1)
counts = np.zeros(numbin+1)
J = np.zeros(numbin+1)


# # Binmesh Calc

# In[6]:


for i in range(0,n):
    index = int((data[i]-origin)/delta)
    if binmesh[index]==0:
        binumber = binumber+1
        J[binumber] = index
        counts[binumber] = 1
        binmesh[index] = binumber
    else:
        counts[int(binmesh[index])] = counts[int(binmesh[index])]+1

midval = []
for ia in range(0,numbin):
    value = ((0.5+ia)*delta) + origin
    midval.append(value)


# # Create Weight

# In[7]:


def CreateWeight(kerneltype,M):
    
    if kerneltype == 1:
        cm = M/((2*M-1)*(n*h))
        wM = [cm for l in range(0,M)]

    if kerneltype == 2:
        cm = 1/(n*h)
        wM = [cm*(1-l/M) for l in range(0,M)]

    if kerneltype == 3:
        cm = 3*(M**2)/((4*(M**2)-1)*(n*h))
        wM = [cm*(1-((l/M)**2)) for l in range(0,M)]

    if kerneltype == 4:
        cm = 0.9375/((1-0.0625/((M**2)))*(n*h));
        wM = [cm*((1-((l/M)**2))**2) for l in range(0,M)]

    if kerneltype == 5:
        part1 = 1 + 0.14583333/(M**4)
        part2 = 0.052083333/(M**6)
        cm = 1.09375/((part1-part2)*(n*h))
        wM = [cm*((1-((l/M)**2))**3) for l in range(0,M)]

    if kerneltype == 6:
        cm = 0.3989*4/(n*h);
        wM = [cm * np.exp(-8*((l/M)**2)) for l in range(0,M)]
    
    return(wM)


# # Weight Bins

# In[8]:


wM = CreateWeight(kernel_type,M)
fh = np.zeros(numbin)

for kw in range(1,binumber+1):   
    for lw in range(1-M,M):
        fh[int(J[kw]+lw)] = fh[int(J[kw]+lw)] + wM[abs(lw)]*counts[kw]


# In[9]:


pd.set_option("display.precision", 10)
result_df = pd.DataFrame(np.array([fh,midval]).transpose(),
            columns=['den','mid']).sort_values('mid')


# In[10]:


result_df.to_csv('resfile.csv')

